part of tags;

final StreamController _onThemeChange = new StreamController.broadcast();
/// A stream of events raised when the current [theme] changes or when the
/// [Theme.refresh] method is called on the current theme.
Stream<Null> get onThemeChange => _onThemeChange.stream;

Theme _theme = new Theme();
/// Returns the current global [Theme].
/// 
/// This getter has the `dynamic` type annotation so that properties of 
/// user-defined [Theme] subclasses may be accessed using dot notation.  For
/// example, consider a user-defined [Resources] class:
/// 
///     class AmazingResources extends Resources {
///       String get style =>
///     """
///     .amazing {
///       color:${theme.colors.amazing};
///     }
///     """;
///       const AmazingResources();
///     }
///
/// Thanks to the `dynamic` type we can refer to `theme.colors.amazing` above
/// without any static analysis getting in our way.
get theme => _theme;

/// Sets the current global [Theme].
/// 
/// The given [theme] must _not_ be `null`.
set theme(Theme theme) {
  assert(theme != null);
  if (_theme != theme) {
    _theme = theme;
    _onThemeChange.add(null);
  }
}

/// A base class for global styling of [Themeable] elements.
class Theme {
  
  /// A set of colors provided by this theme.
  final ThemeColors colors = const ThemeColors();
  
  /// A set of display properties provided by this theme.
  final ThemeDisplay display = const ThemeDisplay();
  
  /// A set of images provided by this theme.
  final ThemeImages images = const ThemeImages();
  
  /// Refresh this theme.
  /// 
  /// Calling this method will add a new event to [onThemeChange] if this theme 
  /// is the current global [theme].  Otherwise it has no effect.
  /// 
  /// This method is designed to be called when one of the properties provided
  /// by this theme changes.  For example, the [fonts.size] may change based on
  /// detection of a user's display properties at runtime or in response to a 
  /// change of user settings.
  void refresh() {
    if (_theme == this) {
      _onThemeChange.add(null);
    }
  }
  
  const Theme();
}

class ThemeColors {
  
  final String background       = '#dedede';
  
  final String backgroundLight  = '#efefef';
  
  String get border             => background;
  
  final String error            = 'red';
  
  final String hover            = 'lightgreen';
  
  final String selection        = 'lightblue';
  
  /// The default text color for this theme.
  final String text             = 'black';
  
  final String textDisabled     = 'grey';
  
  String get textHover          => text;
  
  const ThemeColors();
}

class ThemeColorsDark extends ThemeColors {
  
  final String background       = '#242424';  
  final String backgroundLight  = '#333333';
  final String hover            = '#dc5700';
  final String selection        = '#181818';
  final String text             = '#cecece';
  
  const ThemeColorsDark();
}

class ThemeDark extends Theme {
  
  final ThemeColors colors = const ThemeColorsDark();
  
  const ThemeDark();
}

class ThemeDisplay {
  
  /// The default box size for this theme.
  final String boxSize = '24px';
  
  /// The default font family for this theme.
  final String fontFamily = 'Verdana, Arial, sans-serif';
  
  /// The default font size for this theme, in pixels.
  final int fontSizePx = 12;
  
  String get fontSize => '${fontSizePx}px';
  
  const ThemeDisplay();
}

class ThemeImages {
  const ThemeImages();  
}
