@TestGroup('builder')
library tags.test.builder;

import 'dart:html';
import 'package:tags/tags.dart';
import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart';

Tag elementA = createTag((_) => new CustomElementA());
class CustomElementA extends SpanElement {
  static const TAG = 'custom-a';
  factory CustomElementA() => new Element.tag('span', TAG);  
  CustomElementA.created() : super.created() {
    this.text = 'snarf';
  }
}

Tag elementB = createTag((_) => new CustomElementB());
class CustomElementB extends DivElement {
  static const TAG = 'custom-b';
  factory CustomElementB() => new Element.tag('div', TAG);  
  CustomElementB.created() : super.created() {
    children(this, () {
      elementA();
    });
  }
}

bool isCustomElementsRegistered = false;

@Setup
void setup() {
  if (!isCustomElementsRegistered) {
    isCustomElementsRegistered = true;
    document.registerElement(CustomElementA.TAG, CustomElementA, 
        extendsTag: 'span');
    document.registerElement(CustomElementB.TAG, CustomElementB, 
        extendsTag: 'div');
  }
}

@Test()
void testCreateTagNoArgs() {
  Tag tag = createTag((_) {
    expect(buildHostNode, isNull);
    return new Element.tag('span');
  });
  expect(tag, const isInstanceOf<Tag>());
  expect(tag(), const isInstanceOf<SpanElement>());
}

@Test()
void testCreateTagWithArgs() {
  final o = { 'foo' : 'bar' };
  Tag tag = createTag((args) {
    expect(buildHostNode, isNull);
    expect(args, equals(o));
    return new Element.tag('span');
  });
  expect(tag(null, o), const isInstanceOf<SpanElement>());
}

@Test()
void testNestedCustomElementCreationInBuildScope() {
  var e = div(() {
    elementB();
  });
  expect(e.children.length, equals(1));
  var b = e.children[0];
  expect(b, const isInstanceOf<DivElement>());    
  expect(b.children.length, equals(1));
  var a = b.children[0];
  expect(a, const isInstanceOf<SpanElement>());
  expect(a.text, equals('snarf'));
}

@Test()
void testAppend() {
  SpanElement s = new SpanElement();
  var e = div(() {
    append(s);
  });
  expect(e.children.length, equals(1));
  expect(e.children[0], equals(s));
}

@Test()
void testAppendText() {
  var e = div(() {
    appendText('snarf');
  });
  expect(e.children, isEmpty);
  expect(e.childNodes.length, equals(1));
  expect(e.childNodes[0].nodeType, equals(Node.TEXT_NODE));
  expect(e.childNodes[0], const isInstanceOf<Text>());
  expect(e.childNodes[0].text, equals('snarf'));  
}

@Test()
void testAppendTextShadowRoot() {
  var e;
  div(() {
    e = shadow(() {
      appendText('yo');
    });
  });
  expect(e.children, isEmpty);
  expect(e.childNodes.length, equals(1));
  expect(e.childNodes[0].nodeType, equals(Node.TEXT_NODE));
  expect(e.childNodes[0], const isInstanceOf<Text>());
  expect(e.childNodes[0].text, equals('yo'));
}

@Test()
void testShadow() {
  ShadowRoot s;
  var e = div(() {
    s = shadow(() {
      span();
    });
  });
  expect(e.children, isEmpty);
  expect(e.shadowRoot, equals(s));
  expect(s.children.length, equals(1));
  expect(s.children[0], const isInstanceOf<SpanElement>());
}

@Test()
@ExpectError(isNoSuchMethodError)
void testShadowBuildHostNotElementThrows() {
  div(() {
    shadow(() {
      shadow();
    });
  });
}

@Test()
void testChildren() {
  var e = div();
  children(e, () {
    span();
  });
  expect(e.children.length, equals(1));
  expect(e.children[0], const isInstanceOf<SpanElement>());
}
